@@ -52,9 +52,11 @@ KeepawayPlayer::KeepawayPlayer( SMDPAgent* sa, ActHandler* act, WorldModel *wm,
   WM->setTeamName( strTeamName );
   WM->setNumKeepers( iNumKeepers );
   WM->setNumTakers( iNumTakers ); 
+  WM->setMoveSpeed( 0 );
   WM->setNewEpisode( false );
   WM->setLastAction( UnknownIntValue );
   m_timeLastSay = -5;
+  m_timeStartEpisode = -5;
 
   // create initialisation string
   if( iReconnect != -1 )
@@ -280,8 +282,29 @@ SoccerCommand KeepawayPlayer::keeper()
     SA->endEpisode( WM->keeperReward() );
     WM->setNewEpisode( false );
     WM->setLastAction( UnknownIntValue );
+    m_timeStartEpisode = WM->getCurrentTime();
   }
 
+  // move keepaway region
+  if ( WM->getMoveSpeed() != 0 ) {
+    int tDiff = WM->getCurrentTime() - m_timeStartEpisode + 1;
+    Rect kr = WM->getKeepawayRect();
+    VecPosition adjust( WM->getMoveSpeed() * tDiff, 0 );
+    VecPosition corner( SS->getKeepawayLength() / 2, SS->getKeepawayWidth() / 2 );
+    if ( ( corner + adjust ).getX() > PITCH_LENGTH / 2 )
+      WM->setKeepawayRect( VecPosition( PITCH_LENGTH / 2 - SS->getKeepawayLength(),
+					-SS->getKeepawayWidth() / 2 ),
+			   VecPosition( PITCH_LENGTH / 2,
+					SS->getKeepawayWidth() / 2 ) );
+    else
+      WM->setKeepawayRect( -corner + adjust, corner + adjust );
+  }
+
+  //gjk
+//   if ( WM->getAgentObjectType() == OBJECT_TEAMMATE_1 &&
+//        WM->getCurrentCycle() % 20 == 15 )
+//     cout << WM->getCurrentCycle() << ": " << WM->getKeepawayRect().getPosCenter() << endl;
+
   LogDraw.logCircle( "ball belief", WM->getBallPos(),
 		     1.1, 11, false, COLOR_RED, 
 		     WM->getConfidence( OBJECT_BALL ) );
@@ -410,7 +433,7 @@ SoccerCommand KeepawayPlayer::keeperSupport( ObjectT fastest )
   VecPosition posPassFrom = 
     WM->predictPosAfterNrCycles( OBJECT_BALL, iCycles );
   LogDraw.logCircle( "BallPredict", posPassFrom, 1, 70, true, COLOR_BROWN );
-  soc = getOpenForPassFromInRectangle( SS->getKeepawayRect(), posPassFrom );
+  soc = getOpenForPassFromInRectangle( WM->getKeepawayRect(), posPassFrom );
 
   ObjectT lookObject = chooseLookObject( 0.97 );
 
@@ -420,7 +443,7 @@ SoccerCommand KeepawayPlayer::keeperSupport( ObjectT fastest )
 
   ACT->putCommandInQueue( soc );
   //ACT->putCommandInQueue( turnNeckToObject( lookObject, soc ) );
-  ACT->putCommandInQueue( turnNeckToPoint( SS->getKeepawayRect().getPosCenter(), soc ) );
+  ACT->putCommandInQueue( turnNeckToPoint( WM->getKeepawayRect().getPosCenter(), soc ) );
 
   return soc;
 }
