@@ -644,6 +644,9 @@ bool SenseHandler::analyzeHearMessage( char *strMsg )
     case 'o':                                               // online_coach_
         analyzeCoachMessage( strMsg );
         break;
+    case 'c':
+        analyzeTrainerMessage( strMsg );
+        break;
     case 's':                                               // self
       break;                                                // do nothing
     default:                                                // from direction
@@ -693,6 +696,16 @@ bool SenseHandler::analyzeCoachMessage( char *strMsg )
 
   return true;
 }
+
+bool SenseHandler::analyzeTrainerMessage( char *strMsg )
+{
+  Log.log( 605, "received trainer messages: %s" , strMsg );
+  WM->resetEpisode();
+  WM->setMoveSpeed( Parse::parseFirstDouble( &strMsg ) );
+
+  return true;
+}
+
 /*! This method analyzes the check_ball message that is only received by the
     coach. It sets the information in the Worldmodel what the status of the
     ball is.
@@ -844,8 +857,8 @@ bool SenseHandler::analyzeServerParamMessage( char *strMsg )
   readServerParam( "keepaway_length",          strMsg );
   readServerParam( "keepaway_width",           strMsg );
 
-  SS->setKeepawayRect( SS->getKeepawayWidth(),
-		       SS->getKeepawayLength() );
+  VecPosition corner( SS->getKeepawayLength() / 2, SS->getKeepawayWidth() / 2 );
+  WM->setKeepawayRect( -corner, corner );
 
 
   SS->setMaximalKickDist      ( SS->getKickableMargin() +
