@@ -249,6 +249,10 @@ private:
   int           m_lastAction;
   int           m_timeLastAction;
 
+  // keepaway region
+  Rect          m_keepawayRect;
+  double        m_moveSpeed; 
+
 public:
 
   bool          m_bWasCollision;        /*!<Indicates whether it is collision*/
@@ -880,6 +884,13 @@ public:
   double        keeperReward               (                                 );
   int           keeperStateVars            ( double         state[]          );
 
+  void          setMoveSpeed               ( double         speed            );
+  double        getMoveSpeed               (                                 );
+
+  void          setKeepawayRect            ( VecPosition    pos1,
+					     VecPosition    pos2             );
+  Rect          getKeepawayRect            (                                 );
+
   ////////////////////////// VARIOUS (WorldModel.cc) ///////////////////////////
 
   // constructor
