@@ -129,12 +129,14 @@ int WorldModel::keeperStateVars( double state[] )
 {
   ObjectT PB = getClosestInSetTo( OBJECT_SET_TEAMMATES, OBJECT_BALL );
   if ( !SoccerTypes::isTeammate( PB ) )
-    return 0; // maybe change to != agentObject
+    return 0;
 
-  VecPosition C = SS->getKeepawayRect().getPosCenter();
+  VecPosition C = getKeepawayRect().getPosCenter();
 
   double WB_dist_to_C = getGlobalPosition( PB ).getDistanceTo( C );
-
+  if ( getGlobalPosition( PB ).isBehindOf( C ) )
+    WB_dist_to_C *= -1;
+  
   int numK = getNumKeepers();
   int numT = getNumTakers();
 
@@ -155,20 +157,26 @@ int WorldModel::keeperStateVars( double state[] )
     return 0;
 
   double dist_to_C_K[ numK ];
-  for ( int i = 1; i < numK; i++ )
-    dist_to_C_K[ numK ] = getGlobalPosition( K[ i ] ).getDistanceTo( C );
+  for ( int i = 1; i < numK; i++ ) {
+    dist_to_C_K[ i ] = getGlobalPosition( K[ i ] ).getDistanceTo( C );
+    if ( getGlobalPosition( K[ i ] ).isBehindOf( C ) )
+      dist_to_C_K[ i ] *= -1;
+  }
 
   double dist_to_C_T[ numT ];
-  for ( int i = 1; i < numT; i++ )
-    dist_to_C_T[ numT ] = getGlobalPosition( T[ i ] ).getDistanceTo( C );
+  for ( int i = 0; i < numT; i++ ) {
+    dist_to_C_T[ i ] = getGlobalPosition( T[ i ] ).getDistanceTo( C );
+    if ( getGlobalPosition( T[ i ] ).isBehindOf( C ) )
+      dist_to_C_T[ i ] *= -1;
+  }
 
   double nearest_Opp_dist_K[ numK ];
   VecPosition posPB = getGlobalPosition( PB );
   for ( int i = 1; i < numK; i++ ) {
     VecPosition pos = getGlobalPosition( K[ i ] );
-    for ( int j = 1; j < numT; j++ ) {
+    for ( int j = 0; j < numT; j++ ) {
       double tmp = getGlobalPosition( T[ j ] ).getDistanceTo( pos );
-      if ( j == 1 || tmp < nearest_Opp_dist_K[ i ] ) {
+      if ( j == 0 || tmp < nearest_Opp_dist_K[ i ] ) {
         nearest_Opp_dist_K[ i ] = tmp;
       }
     }
@@ -177,9 +185,9 @@ int WorldModel::keeperStateVars( double state[] )
   double nearest_Opp_ang_K[ numK ];
   for ( int i = 1; i < numK; i++ ) {
     VecPosition pos = getGlobalPosition( K[ i ] );
-    for ( int j = 1; j < numT; j++ ) {
+    for ( int j = 0; j < numT; j++ ) {
       double tmp = posPB.getAngleBetweenPoints( pos, getGlobalPosition( T[ j ] ) );
-      if ( j == 1 || tmp < nearest_Opp_ang_K[ i ] ) {
+      if ( j == 0 || tmp < nearest_Opp_ang_K[ i ] ) {
         nearest_Opp_ang_K[ i ] = tmp;
       }
     }
@@ -205,4 +213,23 @@ int WorldModel::keeperStateVars( double state[] )
 
 
 
+void WorldModel::setMoveSpeed( double speed )
+{
+  m_moveSpeed = speed;
+}
+
+double WorldModel::getMoveSpeed()
+{
+  return m_moveSpeed;
+}
+
+void WorldModel::setKeepawayRect( VecPosition pos1, VecPosition pos2 )
+{
+  m_keepawayRect.setRectanglePoints( pos1, pos2 );
+}
+
+Rect WorldModel::getKeepawayRect()
+{
+  return m_keepawayRect;
+}
 
